<?php

namespace WebsiteBundle\DataFixture\ORM;

use Bazookas\AdminBundle\Entity\CMSUserGroup;
use Bazookas\CronBundle\Entity\Cron;
use Doctrine\Common\DataFixtures\FixtureInterface;
use Doctrine\Common\Persistence\ObjectManager;
use WebsiteBundle\Entity\Test;
use WebsiteBundle\Security\Roles;

class LoadBaseData implements FixtureInterface
{

  public function load(ObjectManager $manager)
  {
    //Add the user groups
    $this->addUserGroups($manager);

    //Add the tests
    $this->addTests($manager);

    //Add the crons
    $this->addCrons($manager);

    $manager->flush();
  }

  private function addUserGroups(ObjectManager $manager) {
    $headResearcherGroup = new CMSUserGroup();
    $headResearcherGroup
      ->setName('Hoofdonderzoekers')
      ->setRoles([
        Roles::ROLE_ADMIN,
        Roles::ROLE_ADMIN_RESEARCHER
      ])
    ;

    $manager->persist($headResearcherGroup);

    $researcherGroup = new CMSUserGroup();
    $researcherGroup
      ->setName('Onderzoekers')
      ->setRoles([
        Roles::ROLE_CMS_USER
      ])
    ;

    $manager->persist($researcherGroup);
  }

  private function addTests(ObjectManager $manager) {
    //Pasat test
    $pasat = new Test();
    $pasat
      ->setName('PASAT')
      ->setType(Test::TYPE_PASAT)
      ->setTutorialText('Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed leo ipsum, iaculis a erat interdum, consequat ullamcorper nisi. Mauris dignissim dolor vel nibh congue, non ornare tortor viverra. Etiam scelerisque est massa, nec consectetur neque commodo ut. Nam sed mollis sapien.')
    ;

    $manager->persist($pasat);

    //Control test
    $control = new Test();
    $control
      ->setName('Controle')
      ->setType(Test::TYPE_CONTROL)
      ->setTutorialText('
Duis varius semper eleifend. Suspendisse a congue sapien. Nulla congue cursus massa at aliquam. Quisque est elit, aliquam eu leo eget, interdum fermentum eros. Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae; Nam non rhoncus orci. Fusce blandit mollis arcu, quis iaculis nulla consequat at.')
    ;

    $manager->persist($control);

    //APASAT test
    $apasat = new Test();
    $apasat
      ->setName('APASAT')
      ->setType(Test::TYPE_APASAT)
      ->setTutorialText('Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed leo ipsum, iaculis a erat interdum, consequat ullamcorper nisi. Mauris dignissim dolor vel nibh congue, non ornare tortor viverra. Etiam scelerisque est massa, nec consectetur neque commodo ut. Nam sed mollis sapien.')
    ;

    $manager->persist($apasat);
  }

  private function addCrons(ObjectManager $manager) {
    //Add inactive user cron
    $inactiveUserCron = new Cron();
    $inactiveUserCron
      ->setCommand('bin/console user:inactive')
      ->setExecutionInterval(5)
      ->setIntervalUnit(Cron::INTERVAL_MINUTES)
      ->setEmail('simon@bazookas.be')
    ;

    $manager->persist($inactiveUserCron);

    //Add session reminder cron
    $sessionReminderCron = new Cron();
    $sessionReminderCron
      ->setCommand('bin/console session:reminder')
      ->setExecutionInterval(5)
      ->setIntervalUnit(Cron::INTERVAL_MINUTES)
      ->setEmail('simon@bazookas.be')
    ;

    $manager->persist($sessionReminderCron);
  }
}
