<?php

namespace WebsiteBundle\PageBuilder;

use Bazookas\AdminBundle\PageBuilder\Base\BaseFormPageBuilder;
use Bazookas\AdminBundle\Repository\Interfaces\ListRepositoryInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Security\Core\Encoder\EncoderFactory;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Entity\Quote;
use WebsiteBundle\Entity\Test;

class ExperimentFormPageBuilder extends BaseFormPageBuilder
{
  /**
   * @param Request $request
   * @return bool
   * @throws \Exception
   */
  public function handleForm(Request $request) {
    $form = $this->getForm();
    if (empty($form)) {
      throw new \Exception('The form is empty in ' . get_class($this) . ' handleForm' );
    }

    $form->handleRequest($request);

    if ($form->isSubmitted() && $form->isValid()) {
      $entityRepo = $this->entityManager->getRepository($this->entityClass);
      if (!$entityRepo instanceof ListRepositoryInterface) {
        throw new \Exception('The entity repository must implement the interface: ' . ListRepositoryInterface::class);
      }

      $experiment = $form->getData();
      if ($experiment->getId() === null) {
        $experiment = $this->assignQuotes($experiment);
      }

      $entityRepo->updateEntities([$experiment]);
      return true;
    } else {
      $this->form = $form;
      return false;
    }
  }

  private function assignQuotes(Experiment $experiment) {
    //get all of the generic quotes and choose some randomly
    $quoteRepo = $this->entityManager->getRepository(Quote::class);
    $quotes = $quoteRepo->findBy(['type' => Quote::TYPE_GENERIC]);

    $allowedQuotes = [];
    $numberOfQuotesToGenerate = $experiment->getTotalSessions() - 1;
    while (count($allowedQuotes) < $numberOfQuotesToGenerate || count($quotes) === 0) {
      $index = rand(0, count($quotes) - 1);

      $quote = $quotes[$index];
      $allowedQuotes[] = $quote;

      //remove the quote from the pool of available quotes
      unset($quotes[$index]);
      $quotes = array_values($quotes);
    }

    //Get all of the master quotes and assign one randomly
    $quoteRepo = $this->entityManager->getRepository(Quote::class);
    $masterQuotes = $quoteRepo->findBy(['type' => Quote::TYPE_MASTER]);

    $index = rand(0, count($masterQuotes) - 1);
    $masterQuote = $masterQuotes[$index];
    $allowedQuotes[] = $masterQuote;

    //set all of the allowed quotes for the experiment
    $experiment->setAvailableQuotes($allowedQuotes);

    return $experiment;
  }

}
