<?php

namespace WebsiteBundle\PageBuilder;

use Bazookas\AdminBundle\PageBuilder\Base\BaseFormPageBuilder;
use Bazookas\AdminBundle\Repository\Interfaces\ListRepositoryInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Security\Core\Encoder\EncoderFactory;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Entity\ExperimentCondition;
use WebsiteBundle\Entity\Test;
use WebsiteBundle\Repository\ParticipantRepository;

class ParticipantFormPageBuilder extends BaseFormPageBuilder
{
  /**
   * @var EncoderFactory
   */
  private $encoderFactory;

  public function __construct(EncoderFactory $encoderFactory) {
    $this->encoderFactory = $encoderFactory;
  }

  public function handleForm(Request $request) {
    $form = $this->getForm();
    if (empty($form)) {
      throw new \Exception('The form is empty in ' . get_class($this) . ' handleForm' );
    }

    $form->handleRequest($request);

    if ($form->isSubmitted() && $form->isValid()) {
      $entityRepo = $this->entityManager->getRepository($this->entityClass);
      if (!$entityRepo instanceof ListRepositoryInterface) {
        throw new \Exception('The entity repository must implement the interface: ' . ListRepositoryInterface::class);
      }

      $entity = $form->getData();

      //encode the password
      $salt = base64_encode(random_bytes(32));
      $passwordEncoder = $this->encoderFactory->getEncoder($entity);

      $plainPassword = $entity->getPlainPassword();
      if (!empty($plainPassword)) {
        $entity
          ->setPassword(
          $passwordEncoder->encodePassword(
            $plainPassword,
            $salt
          ))
          ->setSalt($salt)
        ;
      }

      //assign them to a group
      /**
       * @var Experiment $experiment
       */
      $experiment = $entity->getExperiment();

      if (!empty($experiment) && empty($entity->getCondition())) {
        $experimentConditionRepo = $this->entityManager->getRepository(ExperimentCondition::class);
        $conditions = $experimentConditionRepo->getConditionCount($experiment);

        if (!empty($conditions)) {
          $entity->setCondition($conditions[0]['condition']);
        }
      }

      $entityRepo->updateEntities([$form->getData()]);
      return true;
    } else {
      $this->form = $form;
      return false;
    }
  }
}
