<?php

namespace WebsiteBundle\Services;

use Symfony\Component\Templating\EngineInterface;
use Symfony\Component\Translation\TranslatorInterface;
use WebsiteBundle\Entity\Participant;
use WebsiteBundle\Entity\TestSession;

class EmailService
{

  /**
   * @var \Swift_Mailer
   */
  private $mailer;

  /**
   * @var TranslatorInterface
   */
  private $translator;

  /**
   * @var string
   */
  private $contactUsToEmail;

  /**
   * @var string
   */
  private $from;

  public function __construct(\Swift_Mailer $mailer, TranslatorInterface $translator, string $contactUsTo, string $from, EngineInterface $templateEngine) {
    $this->mailer = $mailer;
    $this->translator = $translator;
    $this->contactUsToEmail = $contactUsTo;
    $this->from = $from;
    $this->templateEngine = $templateEngine;
  }

  public function sendContactUsEmail($data) {
    $subject = $this->translator->trans('site.emails.contactUs.subject', [
      '%name%' => $data['name']
    ]);

    $message = \Swift_Message::newInstance()
      ->setSubject($subject)
      ->setFrom($data['email'])
      ->setTo($this->contactUsToEmail)
      ->setBody('Phone Nr.: '.$data['phoneNumber'] ."\r\n". $data['text'], 'text/plain')
    ;

    $this->mailer->send($message);
  }

  public function sendReminderEmail(TestSession $session, $type = 'before') {
    $participant = $session->getParticipant();

    $subject = $this->translator->trans('site.emails.reminder.'.$type.'.subject');

    $plainText = $this->translator->trans('site.emails.reminder.'.$type.'.body', [
      '%firstName%' => $participant->getName(),
      '%dateTime%' => $session->getScheduled()->format('d/m/Y, H:i')
    ]);

    $message = \Swift_Message::newInstance()
      ->setSubject($subject)
      ->setFrom($this->from)
      ->setTo($participant->getEmail())
      ->setBody($plainText, 'text/plain')
      ->addPart(
        $this->templateEngine->render(
          '@Website/Emails/reminder.html.twig',
          [
            'session' => $session,
            'type' => $type
          ]
        ),
        'text/html'
      )
    ;

    $this->mailer->send($message);
  }

  public function sendForgotPasswordEmail(Participant $participant, $forgotLink) {
    $subject = $this->translator->trans('site.emails.forgotPassword.subject');

    $plainText = $this->translator->trans('site.emails.forgotPassword.body', [
      '%firstName%' => $participant->getName(),
      '%link%' => $forgotLink
    ]);

    $message = \Swift_Message::newInstance()
      ->setSubject($subject)
      ->setFrom($this->from)
      ->setTo($participant->getEmail())
      ->setBody($plainText, 'text/plain')
      ->addPart(
        $this->templateEngine->render(
          '@Website/Emails/forgotPassword.html.twig',
          [
            'firstName' => $participant->getName(),
            'link' => $forgotLink
          ]
        ),
        'text/html'
      )
    ;

    $this->mailer->send($message);
  }
}
