<?php

namespace WebsiteBundle\UserProvider;

use Doctrine\ORM\EntityManager;
use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Security\Core\Exception\UsernameNotFoundException;
use Symfony\Component\Security\Core\Exception\UnsupportedUserException;
use Symfony\Component\Security\Core\User\UserProviderInterface;
use WebsiteBundle\Entity\Participant;

class EmailUserProvider implements UserProviderInterface
{
  protected $entityManager;

  public function __construct(EntityManager $entityManager) {
    $this->entityManager = $entityManager;
  }

  public function loadUserByUsername($fieldValue)
  {
    //Retrieve the user
    $user = $this->getUserBy(array('email' => $fieldValue, 'active' => true));
    if ($user instanceof Participant) {
      return $user;
    }
    throw new UsernameNotFoundException(
      sprintf('Email "%s" does not exist.', $fieldValue)
    );
  }

  public function refreshUser(UserInterface $user)
  {
    if (!$user instanceof Participant) {
      throw new UnsupportedUserException(
        sprintf('Instances of "%s" are not supported.', get_class($user))
      );
    }
    return $this->loadUserByUsername($user->getEmail());
  }

  public function getUserBy(array $conditions) {
    $result = $this->entityManager
      ->getRepository(Participant::class)
      ->findOneBy($conditions)
    ;
    return $result;
  }

  public function supportsClass($class)
  {
    return $class === Participant::class;
  }
}
