<?php

namespace Bazookas\AdminBundle\Entity;

use Bazookas\CommonBundle\Security\Roles;
use Doctrine\ORM\Event\PreUpdateEventArgs;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\Common\Collections\ArrayCollection;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use FOS\UserBundle\Model\GroupInterface;
use Bazookas\CommonBundle\Entity\Base\BaseCMSUser;
use Bazookas\CommonBundle\Entity\Traits\LocalisedTrait;
use Symfony\Component\Security\Core\Authorization\AuthorizationChecker;

  /**
  * @ORM\Table(name="cms_user")
  * @ORM\Entity(repositoryClass="Bazookas\AdminBundle\Repository\CMSUserRepository")
  * @UniqueEntity(
  *     fields={"email"},
  *     message="email.notUnique"
  * )
  * @UniqueEntity(
  *     fields={"username"},
  *     message="username.notUnique"
  * )
  * @ORM\InheritanceType("SINGLE_TABLE")
  * @ORM\DiscriminatorColumn(name="type", type="string")
  * @ORM\HasLifecycleCallbacks
  */
class CMSUser extends BaseCMSUser
{
  use LocalisedTrait;

  /**
  * @ORM\Column(type="string", nullable=true)
  * @Assert\NotBlank()
  */
  protected $locale;

  /**
   * @ORM\ManyToMany(targetEntity="CMSUserGroup")
  */
  protected $groups;

  /**
   * @Assert\NotBlank(groups="registration")
   */
  protected $plainPassword;

  protected $type;

  public function __construct() {
    parent::__construct();
    $this->groups = new ArrayCollection();
    $this->locale = 'nl';
  }

  public function getGroups()
  {
    return $this->groups;
  }

  /**
   * @return string
   */
  public function getType()
  {
    return $this->type;
  }

  /**
   * @ORM\PreUpdate()
   * @param PreUpdateEventArgs $event
   */
  public function updateRoles(PreUpdateEventArgs $event) {
    $roles = [];
    foreach ($this->getGroups() as $group) {
      $roles = array_merge($roles, $group->getRoles());
    }

    // we need to make sure to have at least one role
    $roles[] = static::ROLE_DEFAULT;

    $this->roles = array_unique($roles);
  }

  /**  ------------------------------------------------------
  *                     ACCESS CONTROL
  * ---------------------------------------------------------
  */
  public function canDelete(AuthorizationChecker $auth, BaseCMSUser $user) {
    if ($this->getId() !== $user->getId()) {
      return $auth->isGranted($this->getRoleRequiredForDelete());
    }

    return false;
  }

  /**
  * overwrite of BaseEntity method
  */
  public function getRoleRequiredForView() {
    return Roles::ROLE_ADD_USER;
  }

}
