<?php

namespace Bazookas\AdminBundle\Form;

use Bazookas\AdminBundle\Filter\ListFilter;
use Bazookas\AdminBundle\Filter\ListFilterCollection;
use Bazookas\AdminBundle\Form\Type\NestedFormType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\OptionsResolver\OptionsResolver;

class ListFilterCollectionForm extends AbstractType
{

  private $request;
  private $isPicker;

  public function __construct(RequestStack $requestStack) {
    $this->request = $requestStack->getCurrentRequest();
    $this->isPicker = $this->request->get('picker');
  }

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    /** @var ListFilterCollection $collection */
    $collection = $builder->getData();

    // create an empty filter to act as prototype
    $prototypeData = new ListFilter();
    $prototypeData->setFilterFields($collection->getFilters()[0]->getFilterFields());

    $builder
      ->add('picker', HiddenType::class, [
        'mapped' => false,
        'data' => $this->isPicker,
      ])
      ->add('filters', NestedFormType::class, [
        'label' => 'admin.entities.Filter.fields.name',
        'entry_type' => ListFilterForm::class,
        'entry_options' => [
          'attr' => ['class' => 'box-inline'],
        ],
        'prototype_data' => $prototypeData,
        'class' => 'mar-btm',
        'content_class' => 'js-filter form-inline',
        'inline_actions' => true,
        'confirm_delete' => false,
      ])
    ;
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults([
      'data_class' => ListFilterCollection::class,
      'translation_domain' => 'admin',
      'csrf_protection' => false,
    ]);
  }
}
