<?php
// Stolen and enhanced from old bazookas admin bundles
namespace Bazookas\AdminBundle\Form\Type;

use Symfony\Component\Form\Extension\Core\Type\IntegerType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Bazookas\AdminBundle\Form\DataTransformer\EntityToIDTransformer;
use Doctrine\Common\Persistence\ObjectManager;
use Bazookas\AdminBundle\Util\Copy;

class EntityPickerType extends AbstractType
{
  const DISPLAY_ASSET = 'asset';
  const DISPLAY_IMAGE = 'image';
  const DISPLAY_TEXT = 'text';

  private $objectManager;
  private $transformer;

  public function __construct(ObjectManager $objectManager)
  {
    $this->objectManager = $objectManager;
  }


  public function getParent()
  {
    return IntegerType::class;
  }

  public function getName()
  {
    return 'entity_picker';
  }

  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $transformer = new EntityToIDTransformer($this->objectManager, $options);
    $builder->addModelTransformer($transformer);
    $this->transformer = $transformer;
  }

  public function buildView(FormView $view, FormInterface $form, array $options)
  {
    // NOTE this transformer is re-used for every rendered form item
    // this means
    $view->vars['transformer'] = $this->transformer;
    Copy::theProperties(array(
      'title',
      'icon',
      'button',
      'picker_route',
      'display_field',
      'display_type',
      'class'
    ))
      ->from($options)
      ->to($view->vars);
  }

  public function configureOptions(OptionsResolver $resolver) {
    // See inline comments for better understanding of options
    $resolver->setDefaults(array(
      'class' => null,
      'icon' => 'ti-search', // button icon
      'button' => 'admin.actions.select', // button text
      'title' => 'admin.actions.select', // title of the modal
      'display_field' => 'id', // which field to use as display
      'display_type' => 'text',
      // text: a simple string display
      // image: display image url in img tag (no asset filters)
      // asset: display as image and use imagine_filter('thumb')
      'picker_route' => array(
        'route' => '', // the name of the route to use
        'params' => array(
          'picker' => true, // add picker to url params to be able to identify it as such
        ), // params to pass to the twig `url` function
      ),
      'translation_domain' => 'admin'
    ));

    $resolver->setNormalizer('picker_route', function (OptionsResolver $options, $value) {
      $value['params']['picker'] = true; // make sure it stays when option is given
      return $value;
    });
  }
}
