<?php
namespace Bazookas\AdminBundle\Form\Type;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\CallbackTransformer;
use Symfony\Component\Form\Extension\Core\Type\CollectionType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Bazookas\AdminBundle\Util\Copy;

class NestedFormType extends AbstractType {

  public function buildForm(FormBuilderInterface $builder, array $options) {
    $builder
      ->addModelTransformer(new CallbackTransformer(
        //Transform original value to form value
        function ($value) {
          return $value;
        },
        //Transform form value to original value
        function ($value) {
          if (is_array($value)) {
            return array_values($value);
          } else {
            return $value;
          }
        }
      ))
    ;
  }

  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults(array(
      'by_reference' => false,
      'allow_add' => true,
      'allow_delete' => true,
      'min' => 0,
      'max' => 0, // zero or falsy value means infinite (if no items are allowed, don't add the form item, derp!)
      'sortable' => false,
      'sortable_field' => 'weight',
      'prototype_name' => '__nested__',
      'inline_elements' => false,
      'inline_actions' => false,
      'error_bubbling' => true,
      'add_button_label' => 'admin.nestedForm.add',
      'class' => 'panel nested-form__item',
      'content_class' => 'panel-body',
      'confirm_delete' => true,
      'translation_domain' => 'admin'
    ));
  }


  public function buildView(FormView $view, FormInterface $form, array $options)
  {
    Copy::theProperties(array(
      'prototype_name',
      'sortable',
      'sortable_field',
      'min',
      'max',
      'inline_elements',
      'inline_actions',
      'add_button_label',
      'class',
      'content_class',
      'confirm_delete',
    ))
      ->from($options)
      ->to($view->vars);

    if (isset($view->vars['attr']['class'])) {
      $options['class'] .= $view->vars['attr']['class'];
    }
  }

  public function getParent() {
    return CollectionType::class;
  }
}
