<?php

namespace Bazookas\AdminBundle\Repository;

use Bazookas\AdminBundle\Entity\CMSUser;
use Bazookas\AdminBundle\Repository\Base\BaseRepository;
use Bazookas\AdminBundle\Repository\Interfaces\CMSUserRepositoryInterface;
use Bazookas\CommonBundle\Entity\Base\BaseCMSUser;
use Doctrine\ORM\Query;

class CMSUserRepository extends BaseRepository implements CMSUserRepositoryInterface
{

  /**
   * @deprecated to be removed in v2.
   * Deprecation reason: Let the class metadata decide this through the parent function
   * @return string
   */
  public function getEntityClass() {
    return CMSUser::class;
  }

  /**
   * @param $id
   * @return array
   */
  public function findAllExcluding($id) {
    $qb = $this->createQueryBuilder('u');
    $qb
      ->select('u')
      ->where($qb->expr()->neq('u.id', ':id'))
      ->setParameter('id', $id)
    ;

    return $qb->getQuery()->getResult();
  }

  /**
   * @param array $fields
   * @param array $params
   * @return \Doctrine\ORM\QueryBuilder
   */
  public function createListQueryBuilder(array $fields, array $params) {
    $qb = $this->createQueryBuilder('e');
    $qb
      ->addSelect('groups')
      ->leftJoin('e.groups', 'groups')
    ;

    return $qb;
  }

  /**
   * Get all roles and all inherited roles for a given cms user
   * @param CMSUser $user
   * @param array $roleHierarchy an instance of "%security.role_hierarchy.roles%"
   * @return array
   */
  public function getRolesForUser(CMSUser $user, array $roleHierarchy)
  {
    $userRoles = $user->getRoles();

    for ($i = 0; $i < count($userRoles); $i++) {
      $userRole = $userRoles[$i];
      if (isset($this->roleHierarchy[$userRole])) {
        $userRoles = array_merge($userRoles, $roleHierarchy);
      }
    }

    return array_unique($userRoles);
  }

  /**
   * Get all of the users who have the given roles. Use the $conditionType parameter to indicate if the user should
   * be returned if they have one or more of the roles, or if they have all of the roles.
   * @param array $roles
   * @param int $hydrationMode
   * @param string $conditionType
   * @return array|CMSUser[]
   */
  public function getUsersByRoles(array $roles, $hydrationMode = Query::HYDRATE_ARRAY, $conditionType = 'OR') {
    $qb = $this->createQueryBuilder('users');

    //Add the conditions for the roles
    $conditions = [];
    foreach ($roles as $role) {
      $parameter = ':' . $role;
      $conditions[] = $qb->expr()->like('users.roles', $parameter);
      $qb->setParameter($parameter, '%"' . $role . '"%');
    }

    if (!empty($conditions)) {
      if ($conditionType === 'OR') {
        $qb->andWhere($qb->expr()->orX(...$conditions));
      } else {
        $qb->andWhere($qb->expr()->andX(...$conditions));
      }
    }

    $qb
      ->andWhere($qb->expr()->eq('users.enabled', ':enabled'))
      ->setParameter('enabled', true)
    ;

    return $qb->getQuery()->getResult($hydrationMode);
  }
}
