## AdminElement containers

- [AccordionContainer](#accordioncontainer)
- [BreadcrumbContainer](#breadcrumbcontainer)
- [collapsibleContainer](#collapsiblecontainer)
- [Container](#container)
- [DivContainer](#divcontainer)
- [FlexContainer](#flexcontainer)
- [FlippablePanelContainer](#flippablepanelcontainer)
- [PanelContainer](#panelcontainer)
- [RowContainer](#rowcontainer)
- [TabbedContainer](#tabbedcontainer)
- [TabbedPanelContainer](#tabbedpanelcontainer)
- [TableContainer](#tablecontainer)
- [TreeViewElementContainer](#treeviewelementcontainer)
- [TreeViewPanelContainer](#treeviewpanelcontainer)
- [WellContainer](#wellcontainer)

#### AccordionContainer

This element renders each of its children in a new accordion tab.

In addition to the general AdminElement options the following container specific options can also be given:
```php
[
  'template' => 'BazookasAdminBundle:AdminElements/Containers:accordionContainer.html.twig',
  'class' => 'panel-group accordion',
  'headerLabel' => ''
]
```

Example:
```php
new AccordionContainer([
  new Container([$currentPanels], [
    'navName' => 'admin.dashboards.home.tabs.current.navName',
  ]),
  new Container([$draftPanels], [
    'navName' => 'admin.dashboards.home.tabs.draft.navName',
  ]),
  new Container([$inactivePanels], [
    'navName' => 'admin.dashboards.home.tabs.inactive.navName',
  ])
], [
  'navName' => 'admin.dashboards.home.tabs.all.navName'
]);
```

#### BreadcrumbContainer

Renders a list of breadcrumbs. To be used in association with `BreadcrumbElement` (see [Elements](./elements.md#breadcrumbelement) for more information and examples)

#### CollapsibleContainer

This elements allows its children to be toggled, it in essence, provides a div wrapper for the Bootstrap collapse functionality.

In addition to the general AdminElement options the following container specific options can also be given:
```php
[
      'template' => 'BazookasAdminBundle:AdminElements/Containers:collapsibleContainer.html.twig',
      'initiallyCollapsed' => true,
      'class' => 'collapse',
      'id' => ''
]
```

Example:
```php
new CollapsibleContainer([$formElement], [
  'id' => 'js-inline-add-form'
]);
```

#### Container

This element has no other purpose then rendering its children and should therefore be used to functionally group items within other containers or elements.

```php
new Container($childElements, $options);
```

#### DivContainer

This element wraps its children in a single div tag.

```php
new DivContainer([$child1, $child2]);
```

### FlexContainer

This element places its children in a row with a flexbox layout. This container currently only supports having 4 children.

In addition to the general AdminElement options the following container specific options can also be given:
```php
[
  'template' => 'BazookasAdminBundle:AdminElements/Containers:flexContainer.html.twig',
  'class' => 'row flex-row',
  'defaultColClass' => 'flex-row-item'
]
```

Example:
```php
new FlexContainer([
  new SmallStatisticElement([], [
    'value' => $sessionCount,
    'label' => 'admin.dashboards.home.tabs.labels.sessionCount',
    'icon' => ' fa-clock-o',
    'panelColor' => 'info'
  ]),
  new SmallStatisticElement([], [
    'value' => $participantCount,
    'label' => 'admin.dashboards.home.tabs.labels.participantCount',
    'icon' => 'fa-users',
    'panelColor' => 'primary'
  ]),
  new SmallStatisticElement([], [
    'value' => $activeParticipants,
    'label' => 'admin.dashboards.home.tabs.labels.activeParticipants',
    'icon' => 'fa-user',
    'panelColor' => 'success'
  ]),
  new SmallStatisticElement([], [
    'value' => $inactiveParticipants,
    'label' => 'admin.dashboards.home.tabs.labels.inactiveParticipants',
    'icon' => 'fa-user-o',
    'panelColor' => 'danger'
  ]),
]);
```

#### FlippablePanelContainer

This container accepts 2 children, a button allows the user to switch between the children. Uses an animation to carry out the transition. This container was initially designed to be used together with the [GraphElement](#graphelement) as the first child and the [StatisticOverviewElement](#statisticoverviewelement) as the second child. Its default icons are therefore a pie chart icon and a table icon. While these are the default states it is perfectly possible to use the container with any of the above elements.

In addition to the general AdminElement options the following container specific options can also be given:
```php
[
  'template' => 'BazookasAdminBundle:AdminElements/Containers:flippablePanelContainer.html.twig',
  // see _animations.scss for available animation styles
  'animationClass' => 'flip',
  'icon' => 'fa-pie-chart',
  'iconClass' => 'fa',
  'icon2' => 'fa-list',
  'iconClass2' => 'fa'
]
```

Example:
```php
   $panel1 = new ErrorElement([], [
      'label' => 'the first panel'
   ]);
   $panel2 = new ErrorElement([], [
      'label' => 'the second panel'
   ]);

   $flipPanel = new FlippablePanelContainer([$panel1, $panel2], [
      'label' => $options['panelLabel']
   ]);
```

#### PanelContainer

Renders the children in a panel.

Can be given the options:
```php
  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:Dashboard/Containers:panelContainer.html.twig',
      'class' => 'panel'
    ]);
  }
```

Example:
```php
new PanelContainer(
    new AlertElement([], [
      'message' => 'admin.errors.notEnoughDataToRenderGraph'
    ])
);
```

#### RowContainer

Renders the children in a bootstrap row. A row container can be given a `defaultColClass`. Each of its children can be provided with an option `colClass` this is css class that should be applied to the child's column element (defaults to the `defaultColClass` ).

In addition to the general AdminElement options the following container specific options can also be given:
```php
[
  'template' => 'BazookasAdminBundle:AdminElements/Containers:panelContainer.html.twig',
  'class' => 'panel'
]
```

If the child of a `RowContainer` is given the option `colClass` then the row container will use this as the boostrap column class when rendering the child. If a `defaultColClass` is defined and a child provides a `colClass` than the `colClass` of the child is used when rendering of the element.

Example:
```php
new RowContainer(
  new GraphElement([], [
      'type' => GraphElement::TYPE_RADAR,
      'options' => [
        'scale' => [
          'ticks' => [
            'max' => 100,
          ],
        ],
        'legend' => [
          'display' => false
        ],
        'title' => [
          'display' => true,
          'text' => $graphTitle
        ]
      ],
      'data' => $dataDefinition,
      'colClass' => 'col-md-6'
    ]
  )
);
```

#### TabbedContainer

Renders its children in a tabbed container, each child is given a tabbed view. It supports the following tab positions and orientations:

```php
  //Tab orientation constants
  const ORIENTATION_HORIZONTAL = 'horizontal';
  const ORIENTATION_VERTICAL = 'vertical';

  //Tab location constants
  const LOCATION_LEFT = 'left';
  const LOCATION_RIGHT = 'right';
```

In addition to the general AdminElement options the following container specific options can also be given:
```php
[
  'template' => 'BazookasAdminBundle:AdminElements/Containers:tabbedContainer.html.twig',
  'class' => 'panel',
  'headerLabel' => '',
  'tabOrientation' => self::ORIENTATION_HORIZONTAL,
  'tabLocation' => self::LOCATION_RIGHT
]
```

Children of a `TabbedContainer` can be given the option `NavName` which is the label that will be used to render the navigation element for that child.

Example:
```php
new TabbedContainer([
  $allPanels,
  $draftPanels,
  $currentPanels,
  $inactivePanels
], [
  'headerLabel' => 'admin.dashboards.home.panelHeader',
  'tabOrientation' => TabbedContainer::ORIENTATION_HORIZONTAL,
  'tabLocation' => TabbedContainer::LOCATION_LEFT
]);
```

#### TabbedPanelContainer

Render its children in tabbed panel container, each child is given a tabbed view.

In addition to the general AdminElement options the following container specific options can also be given:
```php
[
  'template' => 'BazookasAdminBundle:AdminElements/Containers:tabbedPanelContainer.html.twig',
  'class' => 'panel',
  'headerLabel' => ''
]
```

Example:
```php
new TabbedPanelContainer($children, ['headerLabel' => 'admin.views.generalStats.top20Rank']);
```

#### TableContainer

Renders its children in a table view

In addition to the general AdminElement options the following container specific options can also be given:
```php
[
  'template' => 'BazookasAdminBundle:AdminElements/Containers:tableContainer.html.twig',
  'class' => 'table',
  'tableHeading' => [],
  'tableData' => []
]
```

Example:
```php

$data = [
  [
    'myUsername',
    1,
    12312,
    3
  ]
];

$element = new TableContainer([], [
  'tableHeading' => [
    'admin.entities.appUser.fields.username',
    'admin.entities.appUser.fields.rank',
    'admin.entities.appUser.fields.rankScore',
    'admin.entities.appUser.fields.currentTier',

  ],
  'tableData' => $data,
  'navName' => $storyModeName,
  ]
);
```

### TreeViewElementContainer

Renders a tree as an indented list (similar to a file explorer).

A TreeViewElementContainer wraps an array of TreeViewElements, each element can in turn have its own TreeViewElement children.

In addition to the general AdminElement options the following container specific options can also be given:
```php
[
  'template' => 'BazookasAdminBundle:AdminElements/Containers:treeViewElementContainer.html.twig',
  'initiallyCollapsed' => false
]
```

Example:
```php
$element = new TreeViewElementContainer([
    new TreeViewElement([
      new MyElement
    ])
]);
```

#### TreeViewPanelContainer
Renders a tree as an indented list (similar to a file explorer). This tree will be wrapped in a panel

In addition to the general AdminElement options the following container specific options can also be given:
```php
[
  'template' => 'BazookasAdminBundle:AdminElements/Containers:treeViewPanelContainer.html.twig'
]
```

Example:
```php

$element = new TreeViewPanelContainer([
  new TreeViewElementContainer([
    new TreeViewElement([
      new MyElement
    ])
  ])
]
);
```

#### WellContainer

Renders its children in a well.

In addition to the general AdminElement options the following container specific options can also be given:
```php
[
  'template' => 'BazookasAdminBundle:AdminElements/Containers:wellContainer.html.twig',
  'class' => 'well bg-gray-light'
]
```

Example:
```php
$item = new TextElement([], [
  'label' => 'some text'
])

new WellContainer($item);
```
