# Form fields

Some of the default symfony form fields are extended and there are some custom form fields.
For more information on the default symfony form fields see http://symfony.com/doc/current/reference/forms/types.html

Most templates are located in `Bazookas\AdminBundle\Resources\views\Form\fields.html.twig` unless stated otherwise.

NOTE: options inherited by parent type are not stated here, look at symfony docs for that.

## EntityPickerType

- renders button which opens a modal
- extends IntegerType
- example usage in mediabundle

### Form builder options

- **class**: (required) the entity class to pick an instance from
- **icon**: (default: 'ti-search') icon to show on the button
- **button**: (default: 'admin.actions.select') translation label to show on the button
- **title**: (default: 'admin.actions.select') translation label to show in modal title
- **display_field**: (default: 'id') which field to use as display, this is NOT the actual form value
- **display_type**: (default: 'text') how to display the value
  - _EntityPickerType::DISPLAY_TEXT_: a simple string display
  - _EntityPickerType::DISPLAY_IMAGE_: display image url in img tag (no asset filters)
  - _EntityPickerType::DISPLAY_ASSET_: display as image and use imagine_filter('thumb')
- **picker_route**: (required) associative array
  - _route_: the name of the route to use
  - _params_: params to pass along with the URL builder

### Implement JsonSerialize

The target (picked) entity must implement the `\JsonSerializable` interface, for example:
```php
    class Media extends BaseEntity implements \JsonSerializable {
      
      ...
      
      public function JsonSerialize()
      {
        $vars = get_object_vars($this);
        $vars['thumbnail'] = $this->getThumbnail();
        return $vars;
      }
    }
```
## HiddenEntityType

- renders an invisible field to link an entity
- uses `Bazookas\AdminBundle\Form\DataTransformer\EntityToIDTransformer`
- extends HiddenType
- forgot why, but it's there

## ImagePreviewType

- mapped = false
- this previews image input in file fields
- uses javascript to load locally added file and preview with image tag
- extends TextType

### Form builder options

- **filter**: (default: 'thumbnail') imagine filter to apply to previously uploaded images
- **previewFileInput**: (required) the DOM id attribute of the form input you want to preview

## NestedFormType

- renders a nested form
- shows add/remove buttons when allowed to
- can be sorted using drag and drop when enabled
- extends CollectionType

### Form builder options

- **min**: (default: 0)
  - minimum amount of items to be in the form
  - FRONTEND check
- **max**: (default: 0)
  - zero or falsy value means infinite
  - if no items are allowed, don't add the form item, derp!
  - FRONTEND check
- **sortable**: false,
  - whether or not the items are sortable
  - *IMPORTANT*
  - add `@ORM\OrderBy({"weight" = "ASC"})` to the annotation of the relation
  - to make sure the order is correct in edit form
- **sortable_field**: 'weight'
  - which field to use for sorting
  - this field must also be added as a HiddenType to the child form
- **inline_elements**: false
  - should the nested form elements be rendered as an inline-block?
- **inline_actions**: false
  - the actions (drag handle, remove button) are shown in the panel-body
- **add_button_label**: 'admin.nestedForm.add'
  - translation label for the add button
- **class**: 'panel nested-form__item'
  - class of the nested form item
  - merged with `['attr']['class']`
  - `js-nested-form-item` will ALWAYS be added in the template
- **content_class**: 'panel-body'
  - class on the inner div
- **confirm_delete**: true
  - when true a "confirm dialog" is shown before removing the item

### Inherit form builder options with new defaults
- **by_reference**: false
- **allow_add**: true
- **allow_delete**: true
- **prototype_name**: '\_\_nested\_\_'
- **error_bubbling**: true

## PanelDividerType

- divides a form into bootstrap panels
- usefull for long forms
- only works with `BazookasAdminBundle:Layout/Partials:_simpleForm.html.twig` or similar form layouts
- automatically sets `mapped` to `false`

### Form builder options

- **label**: rendered as header for the panel

## TranslationFormType

- renders a two-column translation form
- left is "original language"
- right is a tab-view of other languages
- extends CollectionType

### Form builder options
- **require_all_locales**: (default: false) renders form for every locale in frontend

### Inherit form builder options with new defaults
- **by_reference**: false
- **allow_add**: true
- **allow_delete**: true
- **prototype_name**: '\_\_translation\_\_'
- **error_bubbling**: true

### Translated entity implementation

- the translated entity (nested form) MUST have a locale field rendered as a choice
- this is used in the frontend to check which are the possible translations
- and to post the selected language

## YoutubeUploadType

- renders a button to open a modal to upload a youtube video using the javascript sdk
- this is not the input itself
- puts a full youtube url in a target field
- unmapped

### Form builder options

- **targetField**: (required) DOM id of the input field to put the URL in
