bazookas.dashboard = (function () {
  var self = {};

  self.init = function (context) {
    bazookas.main
      .findInContext('.js-graph', context)
      .initElements(initGraphs);

    bazookas.main
      .findInContext('.flippable-panel', context)
      .initElements(initFlippablePanel);

    bazookas.main
      .findInContext('.js-dashboard-calendar', context)
      .initElements(initCalendar);
  };

  function initGraphs() {
    var $this = $(this);
    var options = parseOptions($this.data('options'));

    new Chart($this, {
      type: $this.data('type'),
      data: $this.data('data'),
      options: options
    });
  }

  function parseOptions(options) {
    if (options.tooltips && options.tooltips.callbacks) {
      for (var property in options.tooltips.callbacks) {
        if (options.tooltips.callbacks.hasOwnProperty(property)) {
          if (options.tooltips.callbacks[property]) {
            options.tooltips.callbacks[property] = self[options.tooltips.callbacks[property]];
          }
        }
      }
    }

    if (options.animation && options.animation.onComplete) {
      options.animation.onComplete = self[options.animation.onComplete];
    }
    if (options.animation && options.animation.onProgress) {
      options.animation.onProgress = self[options.animation.onProgress];
    }

    return options;
  }

  /* -------------------------------
   *          Tooltip functions
     ------------------------------- */
  self.percentageTooltip = function(tooltipItem, data) {
    return ' ' + data.datasets[tooltipItem.datasetIndex].data[tooltipItem.index] + "%";
  };

  /* -------------------------------
   *          Animation functions
   ------------------------------- */
  //For use with pie charts
  self.addPercentageToSegment = function(){
    var ctx = this.chart.ctx;
    ctx.font = Chart.helpers.fontString(Chart.defaults.global.defaultFontFamily, 'normal', Chart.defaults.global.defaultFontFamily);
    ctx.textAlign = 'center';
    ctx.textBaseline = 'bottom';

    this.data.datasets.forEach(function (dataset) {
      for (var i = 0; i < dataset.data.length; i++) {
        var model = dataset._meta[Object.keys(dataset._meta)[0]].data[i]._model,
          mid_radius = model.innerRadius + (model.outerRadius - model.innerRadius)/2,
          start_angle = model.startAngle,
          end_angle = model.endAngle,
          mid_angle = start_angle + (end_angle - start_angle)/2;

        var x = mid_radius * Math.cos(mid_angle);
        var y = mid_radius * Math.sin(mid_angle);

        ctx.fillStyle = '#263238';
        ctx.fillText(dataset.data[i]  + "%", model.x + x, model.y + y);
      }
    });
  };

  //for use with bar charts
  self.addPercentageToBar = function(){
    var ctx = this.chart.ctx;
    ctx.font = Chart.helpers.fontString(Chart.defaults.global.defaultFontSize, Chart.defaults.global.defaultFontStyle, Chart.defaults.global.defaultFontFamily);
    ctx.fillStyle = this.chart.config.options.defaultFontColor;
    ctx.textAlign = 'center';
    ctx.textBaseline = 'bottom';
    this.data.datasets.forEach(function (dataset) {
      for (var i = 0; i < dataset.data.length; i++) {
        var model = dataset._meta[Object.keys(dataset._meta)[0]].data[i]._model;
        if (dataset.data[i] !== null && dataset.data[i] !== 0) {
          ctx.fillText(dataset.data[i] + "%", model.x - 1, model.y - 5);
        }
      }
    });
  };

  self.addValueToBar = function(){
    var ctx = this.chart.ctx;
    ctx.font = Chart.helpers.fontString(Chart.defaults.global.defaultFontSize, Chart.defaults.global.defaultFontStyle, Chart.defaults.global.defaultFontFamily);
    ctx.fillStyle = this.chart.config.options.defaultFontColor;
    ctx.textAlign = 'center';
    ctx.textBaseline = 'bottom';
    this.data.datasets.forEach(function (dataset) {
      for (var i = 0; i < dataset.data.length; i++) {
        var model = dataset._meta[Object.keys(dataset._meta)[0]].data[i]._model;
        if (dataset.data[i] !== null && dataset.data[i] !== 0) {
          ctx.fillText(dataset.data[i], model.x, model.y - 5);
        }
      }
    });
  };

  function initFlippablePanel() {
    var $panel = $(this);
    $panel.on('click', '.flippable-panel__button', clickButtonHandler);
  }

  function clickButtonHandler(event) {
    event.preventDefault();
    var $button = $(this),
      $container = $button.closest('.flippable-panel'),
      target = $button.attr('href')
      ;

    $container.find('.flippable-panel__button').removeClass('active');
    $container.find('.flippable-panel__element').removeClass('active');
    $container.find(target).addClass('active');
    $button.addClass('active');

    return false;
  }

  /* -------------------------------
   *          Calendar element
   ------------------------------- */
  function initCalendar() {
    var $this = $(this);
    var $panel = $this.closest('.panel');

    if ($this.hasClass('active')) {
      var monthName = $this.data('month-name');
      $panel.find('.panel-title').text(monthName);

      //set initial button state
      var $nextButton = $panel.find('.js-calendar-next');
      var $prevButton = $panel.find('.js-calendar-previous');

      $panel
        .data('current', $this.attr('id'))
        .find('.js-dashboard-calendar').hide().removeClass('hide')
      ;

      $this.show();

      //Initialize button handlers
      $nextButton.click(handleCalendarNextButton);
      $prevButton.click(handleCalendarPreviousButton)
    }
  }

  function handleCalendarNextButton(event) {
    var $this = $(event.target);
    var $panel = $this.closest('.panel');
    var $current = $('#'+$panel.data('current'));
    var $next = $current.next();

    if ($next.length > 0) {
      toggleCalendarPanel($this, $current, $next, $panel);
    } else {
      $this.prop('disabled', true);
    }
  }

  function handleCalendarPreviousButton(event) {
    var $this = $(event.target);
    var $panel = $this.closest('.panel');
    var $current = $('#'+$panel.data('current'));
    var $prev = $current.prev();

    if ($prev.length > 0) {
      toggleCalendarPanel($this, $current, $prev, $panel);
    } else {
      $this.prop('disabled', true);
    }
  }

  function toggleCalendarPanel($this, $current, $newElement, $panel) {
    if ($panel.data('fading')) {
      var $fading = $('#'+$panel.data('fading'));
      $fading.stop(true, false).hide();
    }

    $current.fadeOut(function() {
      $newElement.fadeIn();
      $panel.removeData('fading');
    });

    var $btnGroup = $this.closest('.btn-group');
    if ($newElement.next().length > 0) {
      $btnGroup.find('.js-calendar-next').prop('disabled', false);
    } else {
      $btnGroup.find('.js-calendar-next').prop('disabled', true);
    }

    if ($newElement.prev().length > 0) {
      $btnGroup.find('.js-calendar-previous').prop('disabled', false);
    } else {
      $btnGroup.find('.js-calendar-previous').prop('disabled', true);
    }

    $panel.data('fading', $current.attr('id'));
    $panel.data('current', $newElement.attr('id'));

    var monthName = $newElement.data('month-name');
    $panel.find('.panel-title').text(monthName);
  }

  return self;
})();
