## OutputFormatter function reference and exmaples

### Default field transformation functions

 - [propertyToScalar](#propertytoscalar)
 - [propertyToScalarArray](#propertytoscalararray)

#### propertyToScalar

config:

```php
$config = [
  'someField' => [
        'transformation' => [OutputFormatter::class, 'propertyToScalar'],
        'params' => ['id']
  ]
];

```

converts:

 ```json
{
    "someField": {
      "id": 1
    }
}
 ```

to 

 ```json
{
    "someField": 1
}
 ```

Note: the `id` parameter in `params` is the default value but can be overwritten by supplying it in the `params`.

#### propertyToScalarArray

**Deprecated**: use propertyToScalar instead.

config:

```php
$config = [
  'someField' => [
    'transformation' => [OutputFormatter::class, 'propertyToScalarArray'],
    'params' => ['id']
  ]
];

```

converts:

 ```json
{
    "someField": [
      { "id": 1 },
      { "id": 2 }
    ]
}
 ```

to 

 ```json
{
    "someField": [1, 2]
}
 ```
 
 Note: the `id` parameter in `params` is the default value but can be overwritten by suplying it in the `params`.
----------------------------------------------
### Default global transformation functions

 - [addToParent](#addtoparent)
 - [unsetFields](#unsetfields)
 - [convertArrayToSingleton](#convertarraytosingleton)
 - [usePropertyValueAsKey](#usepropertyvalueaskey)
 - [transformFieldKeys](#transformfieldkeys)
 - [renameKey](#renameKey)
 - [addTranslationsToParent](#addtranslationstoparent)

#### addToParent

config:

```php
$globalConfig = [
  [ 
    'transformation' => [OutputFormatter::class, 'addToParent'],
    'params' => 'test'
  ]
];

```

converts:

 ```json
{
    "someField": [1, 2],
    "test": {
          "hi": 1 
     }
}
 ```

to 

 ```json
{
    "someField": [1, 2],
    "hi": 1 
}
 ```

#### unsetFields

Removes a field.

config:

```php
$globalConfig = [
    [
      'transformation' => [OutputFormatter::class, 'unsetFields'],
      'params' => ['created']
    ],
];

```

converts:

 ```json
{
    "someField": 1,
    "created":{
         "date":"2016-12-06 19:07:58.000000",
         "timezone_type":3,
         "timezone":"Europe\/Zurich"
    }
}
 ```

to 

 ```json
{
    "someField": 1
}
 ```

Note that it is also possible to remove more than one field at a time by changing the config to:

```php
$globalConfig = [
    [
      'transformation' => [OutputFormatter::class, 'unsetFields'],
      'params' => [['created', 'anotherField']]
    ],
];
```

#### convertArrayToSingleton

Convert an array to a single object

config:

```php
$globalConfig = [
    [
      'transformation' => [OutputFormatter::class, 'convertArrayToSingleton'],
      'params' => ['someField']
    ],
];

```

converts:

 ```json
{
    "someField": [
      {
        "id": 1
      }
    ]
}
 ```

to 

 ```json
{
    "someField": {
      "id": 1
    }
}
 ```

#### usePropertyValueAsKey

config:

```php
$globalConfig = [
  [
    'transformation' => [OutputFormatter::class, 'usePropertyValueAsKey'],
    'params' => ['type']
  ]
];

```

converts:

 ```json
{
    "IntroStoryData": [
        {
            "Type": "PreIntroduction",
            "PreTextList": [
                  "hi",
                  "this",
                  "is",
                  "a",
                  "test"
                ],
            "Id": 1
        }
    ]
}
 ```

to 

 ```json
{
    "IntroStoryData": {
       "PreIntroduction": {
           "PreTextList": [
                 "hi",
                 "this",
                 "is",
                 "a",
                 "test"
               ],
           "Id": 1
       }
    }
}
 ```

#### transformFieldKeys

config:

```php
$globalConfig = [
  [
    'transformation' => [OutputFormatter::class, 'transformFieldKey'],
    'params' => 'ucfirst'
  ]
];

```

converts:

 ```json
{
    "someField": 1
}
 ```

to 

 ```json
{
    "SomeField": 1
}
 ```

#### renameKey

config:

```php
$globalConfig = [
  [
    'transformation' => [OutputFormatter::class, 'renameKey'],
    'params' => [['someField' => 'anotherField']]
  ]
];

```

converts:

 ```json
{
    "someField": 1
}
 ```

to 

 ```json
{
    "anotherField": 1
}
 ```

#### addTranslationsToParent

data:
```php
$data = [
  'id' => 2,
  'details' => [
    [
      'name' => 'test',
      'description' => 'a description',
      'locale' => 'nl'
    ],
    [
      'name' => 'test',
      'description' => 'a description',
      'locale' => 'fr'
    ]
  ]
];
```

config:

```php
$formatter = new OutputFormatter([], [
  [
    'transformation' => [OutputFormatter::class, 'addTranslationsToParent'],
    'params' => ['details']
  ]
]);
```

converts: 

```json
{
    "id": 1,
    "details": [
      {
        "name": "test",
        "description": "a description",
        "locale": "nl"    
      },
      {
          "name": "test",
          "description": "a description",
          "locale": "fr"    
       }
    ]
}
```

to: 

```json
{
    "id": 1,
    "name": {
      "nl": "test",
      "fr": "test"
    },
    "description": {
      "nl": "test",
      "fr": "test"
    }
}
```
