<?php

namespace Bazookas\CommonBundle\Entity\Base;

use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Security\Core\User\EquatableInterface;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\MappedSuperclass
 */
abstract class BaseUser extends BaseEntity implements UserInterface, EquatableInterface
{

  /**
   * @ORM\Column(type="array", nullable=true)
   */
  protected $roles;

  /**
   * @ORM\Column(type="string")
   * @var string
   */
  private $username;

  /**
   * @ORM\Column(type="string")
   * @var string
   */
  private $password;

  /**
   * @ORM\Column(type="string")
   * @var string
   */
  private $salt;

  public function __construct($username, $password, $salt, array $roles)
  {
    parent::__construct();
    $this->username = $username;
    $this->password = $password;
    $this->salt = $salt;
    $this->roles = $roles;
  }

  /**
   * @return array
   */
  public function getRoles()
  {
    return $this->roles;
  }

  /**
   * @param array $roles
   * @return $this
   */
  public function setRoles(array $roles)
  {
    $this->roles = $roles;

    return $this;
  }

  /**
   * @return string
   */
  public function getPassword()
  {
    return $this->password;
  }

  /**
   * @param string $password
   * @return $this
   */
  public function setPassword(string $password)
  {
    $this->password = $password;

    return $this;
  }

  /**
   * @return string
   */
  public function getSalt()
  {
    return $this->salt;
  }

  /**
   * @param string $salt
   * @return $this
   */
  public function setSalt(string $salt = null)
  {
    $this->salt = $salt;

    return $this;
  }

  /**
   * @return string
   */
  public function getUsername()
  {
    return $this->username;
  }

  /**
   * @param string $name
   * @return $this
   */
  public function setUsername(string $name)
  {
    $this->username = $name;

    return $this;
  }

  public function eraseCredentials()
  {
    //TODO implement this
  }

  /**
   * @param UserInterface $user
   * @return bool
   */
  public function isEqualTo(UserInterface $user)
  {
    if (!$user instanceof BaseUser) {
      return false;
    }

    if ($this->password !== $user->getPassword()) {
      return false;
    }

    if ($this->salt !== $user->getSalt()) {
      return false;
    }

    if ($this->username !== $user->getUsername()) {
      return false;
    }

    return true;
  }
}
