<?php

namespace Bazookas\CommonBundle\Entity\Traits;

use Bazookas\CommonBundle\Entity\Interfaces\LocalisedEntityInterface;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use Doctrine\Common\Collections\ArrayCollection;
use Bazookas\CommonBundle\Entity\Interfaces\EntityDetailInterface;


/**
 * This trait provides the basic functionality for an inverse side details association (an entity containing the translated fields of a parent entity)
 */
trait DetailParentTrait
{

  /**
   * @return EntityDetailInterface[]
   */
  public function getDetails() {
    return $this->details;
  }

  /**
   * @param string $locale
   * @return null|EntityDetailInterface
   * @throws \Exception
   */
  public function getDetail(string $locale) {
    foreach($this->details as $detail) {
      if (!$detail instanceof LocalisedEntityInterface) {
        throw new \Exception('The entity \''.get_class($detail).'\'must implement the interface \'' . LocalisedEntityInterface::class . '\'');
      }

      if ($detail->getLocale() === $locale) {
        return $detail;
      }
    }

    return null;
  }

  /**
   * @param $details
   * @return $this
   */
  public function setDetails($details) {
    if (!$details instanceof ArrayCollection) {
      if (is_object($details) && method_exists($details, 'toArray')) {
        $details = $details->toArray();
      }

      $details = new ArrayCollection($details);
    }

    $this->details = $details;
    return $this;
  }

  /**
   * @param EntityDetailInterface $detail
   * @return $this
   */
  public function addDetail(EntityDetailInterface $detail) {
    $this->details->add($detail);
    if (method_exists($detail, 'setDetailParent')) {
      $detail->setDetailParent($this);
    }

    return $this;
  }

  /**
   * @param EntityDetailInterface $detail
   * @return $this
   */
  public function removeDetail(EntityDetailInterface $detail) {
    $this->details->removeElement($detail);
    if (method_exists($detail, 'setDetailParent')) {
      $detail->setDetailParent(null);
    }

    return $this;
  }

  /**
   * @return ArrayCollection
   */
  public function getDetailLocales() {
    $details = $this->getDetails();

    $locales = new ArrayCollection();
    foreach($details as $detail) {
      $locale = $detail->getLocale();
      $locales->add($locale);
    }

    return $locales;
  }

  /**
   * @param $locale
   * @return mixed
   */
  public function getDetailsByLocale($locale) {
    foreach($this->details as $detail) {
      // NOTE doctrine/symfony is smart enough to compare instances of the same entity and return true
      $detailLocale = $detail->getLocale();
      if ($detailLocale === $locale || (
          method_exists($detailLocale, 'getLocale') && $detailLocale->getLocale() === $locale
        )) {
        return $detail;
      }
    }
    return $detail;
  }

  /**
   * @return bool
   */
  private function checkHasDetail() {
    return !$this->details->isEmpty();
  }

}
