<?php

namespace Bazookas\CommonBundle\Enums\Base;

use ReflectionClass;

/**
 * Class BaseEnum
 * @package Bazookas\CommonBundle\Enums\Base
 * @see https://stackoverflow.com/questions/254514/php-and-enumerations
 */
abstract class BaseEnum
{
  /**
   * An array to cache the constants for each class
   * @var array|null
   */
  private static $constCacheArray = null;

  private function __construct()
  {
    /*
      Prevents creation of an instance of this class
    */
  }

  /**
   * Is this a valid name of a constant
   * @param $name
   * @return bool
   */
  public static function isValidName($name)
  {
    return array_key_exists($name, self::getConstants());
  }

  /**
   * @return array
   */
  public static function getConstants()
  {
    if (self::$constCacheArray == null) {
      self::$constCacheArray = [];
    }
    $calledClass = get_called_class();
    if (!array_key_exists($calledClass, self::$constCacheArray)) {
      $reflect = new ReflectionClass($calledClass);
      self::$constCacheArray[$calledClass] = $reflect->getConstants();
    }

    return self::$constCacheArray[$calledClass];
  }

  /**
   * Is the value a valid constant value
   * @param $value
   * @return bool
   */
  public static function isValidValue($value)
  {
    $values = array_values(self::getConstants());

    return in_array($value, $values, $strict = true);
  }

  /**
   * Return the name of the constant for the given value
   * @param $value
   * @param bool $strict
   * @return false|int|string
   */
  public static function getNameForValue($value, $strict = true)
  {
    return array_search($value, self::getConstants(), $strict);
  }

  /**
   * Get the allowed values for the defined constants
   * @return array
   */
  public static function getAllowedValues()
  {
    return array_values(self::getConstants());
  }
}
