### Formatting Utility

Welcome to the wonderous wonderful wonder world of Formatting Utility! If you notice any dragons, odd happenings or magic please inform the nearest attendant using the `git issue` system. 

The Formatting Utility is designed to convert `EntityInterface` objects to associative arrays. **Note** that this utility will only convert `EntityInterface` objects, all other objects are left as-is. For example let's assume we have the following entity definition:

```php
<?php

namespace Bazookas\CommonBundle\Tests\Bazookas\CommonBundle\Entity;

use Bazookas\CommonBundle\Entity\Base\BaseEntity;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 */
class SimpleTestEntity extends BaseEntity {

  /**
  * @var string
  */
  private $name;

  /**
  * @var \DateTime
  */
  private $created;
  
  /**
  * @ORM\ManyToOne(targetEntity="YetAnotherEntity")
  */
  private $anotherEntity;
}

```

In the above entity the `FormattingUtility` will convert the `anotherEntity` field to an associative array. However the `created` field is left as-is. This allows the formatting utility to output an associative array in a format fairly similar to the doctrine array hydration output thus making it much easier to use in conjunction with your existing `OutputFormatter`s.

You can use the `FormattingUtility` as follows:

```php
$awards = []; //An array of Award entity objects
$awards = FormattingUtility::entityToArray($awards);
```

This will output something along the lines of:

![formatting-utility-dump-1.png](/Resources/docs/Images/formatting-utility-dump-1.png)

There are a couple of important things to note here:

- If a function will return a doctrine `Proxy` instance or an uninitialised instance of `PersistenceCollection` than the field is ignored.
- If an entity contains circular reference, the `FormattingUtility` will replace these with an `object-hash-` reference. These references are populated with an arrayified instance of the object where possible. You will rarely need to have access to these circular reference items, however in the case that you do need them you can make use of the `FormattingUtility::getObjectByHash` function. This function will return the arrayified object for the given hash id. **Important note**: The `FormattingUtility` works by storing a hash map of the objects it converts, every time you call `FormattingUtility::entityToArray` this hash map will be cleared. For this reason it is also possible to retrieve the entire hash map using the `getObjectHashMap` function. Additionally since the object hash map will continue to exist until the next invocation of the `entityToArray` function a helper function also exists to clear the hash map `clearObjectHashMap` for memory clearing purposes.
