# Email template

This bundle contains a simple email template. It's basically a wrapper to which you can pass colors and fonts.

## Available customisations

### Options (and the default values if any)

#### General

- subject: (required, placed in title for view in browser)
- header: (will be overwritten by the header block)
- body: (will be overwritten by the body block)
- footer: (will be overwritten by the footer block)

#### Layout and size
- bodyPadding: 10
- contentWidth: 600
- contentPadding: 10

#### Background and text
- background: '#EEEEEE'
- contentBackground: '#FFFFFF'
- textColor: '#000000'
- contentTextColor: (inherits textColor)
- defaultTextAlign: 'left'
- font: 'Helvetica, Arial, sans-serif'

### Blocks

- header
- contentTable (overwrite this if you want a special layout)
  - content (use for a simple 1 column email)
- footer

The header and footer are placed outside of the contentTable to render "view in browser" links or disclaimers. You can also use the header for a super fancy logo or something. What-evah!

## How to use

### Embed

By using the embed tag you can override the default option using the `with` keyword. All options are listed above.

```twig
{% embed 'BazookasCommonBundle:Layout:email.html.twig' with {
  defaultTextAlign: 'center',
} %}

  {% trans_default_domain 'emails' %}
  {% block header %}
    <p style="font-size: 9px;text-align:center;">{{ 'email.general.notReadable'|trans }} <a href="#">{{ 'email.general.viewInBrowser'|trans }}</a></p>
  {% endblock %}

  {% block content %}
    <h3 style="margin-top:0;">{{ 'email.invite.title'|trans({'%sender%': sender}) }}</h3>
    <p>{{ 'email.invite.content'|trans|nl2br }}</p>
  {% endblock %}

  {% block footer %}
    <p style="font-size: 9px;text-align:center;">{{ 'email.general.footer'|trans }}</p>
  {% endblock %}

{% endembed %}
```

### Extend

You can also extend the email template. You can also overwrite default options by passing them from with the template data from the controller or defining them in the template itself (as seen below).

```twig
{% extends 'BazookasCommonBundle:Layout:email.html.twig' %}

{% trans_default_domain 'emails' %}
{% block header %}
  <p style="font-size: 9px;text-align:center;">{{ 'email.general.notReadable'|trans }} <a href="#">{{ 'email.general.viewInBrowser'|trans }}</a></p>
{% endblock %}

{% block content %}
  <h3 style="margin-top:0;">{{ 'email.invite.title'|trans({'%sender%': sender}) }}</h3>
  <p>{{ 'email.invite.content'|trans|nl2br }}</p>
{% endblock %}

{% block footer %}
  <p style="font-size: 9px;text-align:center;">{{ 'email.general.footer'|trans }}</p>
{% endblock %}
```

### In php (controller or service)

If you cannot or don't want to make a custom template, you can also pass raw html to the `header`, `footer` and `body` options as part of the template data of the render function. These will be rendered as raw html in the blocks of the same name (if not overwritten by child templates).

Example:

```php
    // NOTE ugly, just an example
    return $this->render('@BazookasCommon/Layout/email.html.twig', [
      'subject' => 'test email',
      'background' => 'black',
      'textColor' => 'white',
      'contentTextColor' => 'black',
      'header' => '<p style="font-size:9px;text-align:center;">some small header text</p>',
      'body' => 'Raw html body here',
      'footer' => '<p style="font-size:9px;text-align:center;">some small footer text</p>',
    ]);
```
