<?php

namespace Bazookas\CommonBundle\Utility;

class Utility {

  /**
   * Check if a string $haystack starts with the given $needle
   * @param string $haystack
   * @param string $needle
   * @return bool
   */
  public static function startsWith(string $haystack, string $needle) {
    // search backwards starting from haystack length characters from the end
    return !empty($needle) ? strrpos($haystack, $needle, -strlen($haystack)) !== false : false;
  }

  /**
   * Check if a string $haystack ends with the given $needle
   * @param string $haystack
   * @param string $needle
   * @return bool
   */
  public static function endsWith(string $haystack, string $needle) {
    // search forward starting from end minus needle length characters
    return !empty($needle) ? (($temp = strlen($haystack) - strlen($needle)) >= 0 && strpos($haystack, $needle, $temp) !== false) : false;
  }

  /**
   * Generate a random string based of length $length and containing the characters $characters
   * @param int $length
   * @param string $characters
   * @return string
   */
  public static function generateRandomString(int $length = 16, string $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ') {
    $charactersLength = strlen($characters) - 1;
    $randomString = '';
    for ($i = 0; $i < $length; $i++) {
        $randomString .= $characters[rand(0, $charactersLength)];
    }
    return $randomString;
  }

  /**
   * Filter an array recursively using the provided callback
   * @param array $input
   * @param callable $callback
   * @return array
   */
  public static function arrayFilterRecursive(array &$input, callable $callback) {
    foreach ($input as &$value) {
      if (is_array($value)) {
        $value = self::arrayFilterRecursive($value, $callback);
      }
    }

    return array_filter($input, $callback);
  }

  /**
   * Determine if an array is associative or numerically indexed
   * @param array $array
   * @return bool
   */
  public static function isAssociative(array $array)
  {
    return count(array_filter(array_keys($array), 'is_string')) > 0;
  }

  /**
   * Convert seconds into human readable time.
   * @param int|string $value
   * @return array|false
   */
  public static function convertTimeToReadable($value) {
    $value = intval($value);

    if (!empty($value) && $value > 0) {
      $seconds = $value % 60;
      $minutes = floor($value / 60) % 60;
      $hours = floor($value / 3600) % 24;
      $days = intval(floor($value / 86400));

      return [$days, $hours, $minutes, $seconds];
    } else {
      return false;
    }
  }

  /**
   * @param int[]|float[] $data a sorted array containing numeric values
   * @return int|float
   */
  public static function getMedianValue(array &$data) {
    if (empty($data)) {
      return 0;
    }

    $arrayCount = count($data);
    $medianIndex = (int)($arrayCount / 2);

    //Is it an even number?
    if ($arrayCount % 2 == 0) {
      return (($data[$medianIndex-1] + $data[$medianIndex]) / 2);
    } else {
      return $data[$medianIndex];
    }
  }

  /**
   * Check if an array contains only empty values
   * @param array $data
   * @return bool
   */
  public static function arrayContainsOnlyEmptyValues(array &$data) {
    $emptyCount = 0;
    foreach($data as &$item) {
      if (empty($item)) {
        $emptyCount++;
      }
    }

    return $emptyCount === count($data);
  }

}
