<?php

namespace Bazookas\MediaBundle\EventListeners;

use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Event\PreUpdateEventArgs;
use Bazookas\MediaBundle\Services\Media\Image\ImageService;
use Symfony\Component\Filesystem\Filesystem;
use Bazookas\MediaBundle\Entity\Image;
use Bazookas\MediaBundle\Entity\Video;
use Bazookas\MediaBundle\Entity\Interfaces\MediaEntityInterface;
use Bazookas\MediaBundle\Services\Media\Video\VideoServiceChain;

class MediaUploadListener {

  private $imageService;
  private $videoServices;

  public function __construct(ImageService $imageService, VideoServiceChain $videoServiceChain )
  {
    $this->imageService = $imageService;
    $this->videoServices = $videoServiceChain->getVideoServices();
  }

  public function prePersist(LifecycleEventArgs $args)
  {
    $entity = $args->getEntity();
    if ($entity instanceof MediaEntityInterface) {
      $this->uploadFile($entity, $args);
    }
  }

  public function preUpdate(PreUpdateEventArgs $args)
  {
    $entity = $args->getEntity();
    if ($entity instanceof MediaEntityInterface) {
      $this->uploadFile($entity, $args);
    }
  }

  public function uploadFile(MediaEntityInterface $mediaItem, LifecycleEventArgs $args) {
    $entityManager = $args->getEntityManager();

    switch(true) {
      case $mediaItem instanceof Image:
        $this->imageService->setEntityManager($entityManager);
        $this->imageService->handleUpload($mediaItem->getFile(), $mediaItem, $mediaItem->getDirectory());
        break;
      case $mediaItem instanceof Video:
        $this->handleVideoUpload($args, $mediaItem->getFile(), $mediaItem, $mediaItem->getDirectory(), $entityManager);
        break;
    }
  }

  private function handleVideoUpload(LifecycleEventArgs $args, $file, MediaEntityInterface $mediaItem, $directory, $entityManager) {
    $service = null;
    foreach($this->videoServices as $videoService) {
      if ($videoService->getProvider() === $mediaItem->getProvider()) {
        $videoService->setEntityManager($entityManager);
        return $videoService->handleUpload($args, $mediaItem->getFile(), $mediaItem, $mediaItem->getDirectory());
      }
    }

    //TODO replace this with a custom, catchable exception
    throw new \Exception('No service found for the provider: \''.$mediaItem->getProvider().'\'');
  }

}
