<?php

namespace Bazookas\MediaBundle\Form;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Bazookas\MediaBundle\Form\Type\MediaFolderType;
use Bazookas\MediaBundle\Entity\MediaFolder;
use Symfony\Component\Security\Core\Authorization\AuthorizationChecker;

class MediaFolderAdminType extends AbstractType
{

  private $auth;
  private $defaultFolders;

  public function __construct(AuthorizationChecker $auth, $defaultFolders) {
    $this->auth = $auth;
    $this->defaultFolders = $defaultFolders;
  }

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $builder
      ->add('name', TextType::class, array(
        'label' => 'media.entities.mediaFolder.fields.name'
      ))
      ->add('parent', MediaFolderType::class, array(
        'label' => 'media.entities.mediaFolder.fields.parent'
      ))
    ;

    if ($this->auth->isGranted('ROLE_SUPER_ADMIN')) {
      $builder
        ->add('type', ChoiceType::class, array(
          'label' => 'media.entities.mediaFolder.fields.type',
          'choices' => array(
            'media.entities.mediaFolder.enums.TYPE_FOLDER' => MediaFolder::TYPE_FOLDER,
            'media.entities.mediaFolder.enums.TYPE_HIDDEN_FOLDER' => MediaFolder::TYPE_HIDDEN_FOLDER,
          ),
        ))
      ;

      $defaultFolderChoices = $this->getDefaultFolderChoices();
      if (!empty($defaultFolderChoices)) {
        $builder
          ->add('defaultFolder', ChoiceType::class, array(
            'label' => 'media.entities.mediaFolder.fields.defaultFolder',
            'choices' => $defaultFolderChoices,
            'required' => false
          ))
        ;
      }
    }
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults(array(
      'data_class' => MediaFolder::class
    ));
  }

  private function getDefaultFolderChoices() {
    $choices = [];

    foreach($this->defaultFolders as $key => $default) {
      $choices['media.entities.mediaFolder.enums.'.$key] = $default;
    }

    return $choices;
  }
}
